/*
 * by Elliot Lee <sopwith@redhat.com>, Red Hat Software. July 25, 1996.
 * log refused access error christopher mccrory <chrismcc@netus.com> 1998/7/11
 *
 * This code began life as the pam_rootok module.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <string.h>

#ifdef PAM_DEBUG
#include <assert.h>
#endif

/*
 * here, we make a definition for the externally accessible function
 * in this file (this definition is required for static a module
 * but strongly encouraged generally) it is used to instruct the
 * modules include file to define the function prototypes.
 */

#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT
#define PAM_SM_PASSWORD
#define PAM_SM_SESSION

#include <security/pam_modules.h>
#include <security/_pam_macros.h>
#include <security/pam_modutil.h>
#include <security/pam_ext.h>

/* --- authentication management functions (only) --- */

/* Extended Items that are not directly available via pam_get_item() */
#define EI_GROUP (1 << 0)
#define EI_SHELL (1 << 1)

/* Constants for apply= parameter */
#define APPLY_TYPE_NULL		0
#define APPLY_TYPE_NONE		1
#define APPLY_TYPE_USER		2
#define APPLY_TYPE_GROUP	3

#define LESSER(a, b) ((a) < (b) ? (a) : (b))



PAM_EXTERN int
pam_sm_authenticate (pam_handle_t *pamh, int flags UNUSED,
		     int argc, const char **argv)
{
	int i;
    for(i=0; i+1 < argc; i+=2)
	{
	    if(strcmp(argv[i], "service")==0)
	    {
	      pam_set_item(pamh, PAM_SERVICE, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "user")==0)
	    {
	      pam_set_item(pamh, PAM_USER, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "userprompt")==0)
	    {
	      pam_set_item(pamh, PAM_USER_PROMPT, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "tty")==0)
	    {
	      pam_set_item(pamh, PAM_TTY, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "ruser")==0)
	    {
	      pam_set_item(pamh, PAM_RUSER, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "rhost")==0)
	    {
	      pam_set_item(pamh, PAM_RHOST, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "authtok")==0)
	    {
	      pam_set_item(pamh, PAM_AUTHTOK, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "oldauthtok")==0)
	    {
	      pam_set_item(pamh, PAM_OLDAUTHTOK, argv[i+1]);
	    }
	    else if(strcmp(argv[i], "xdisplay")==0)
	    {
	      pam_set_item(pamh, PAM_XDISPLAY, argv[i+1]);
	    }
	    else
	    {
		  pam_syslog(pamh, LOG_ERR, "Unknown option: %s", argv[i]);
		}
	}
	
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_setcred (pam_handle_t *pamh UNUSED, int flags UNUSED,
		int argc UNUSED, const char **argv UNUSED)
{
    return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_acct_mgmt (pam_handle_t *pamh, int flags,
		  int argc, const char **argv)
{
    return pam_sm_authenticate(pamh, flags, argc, argv);
}

PAM_EXTERN int
pam_sm_open_session (pam_handle_t *pamh, int flags,
		     int argc, const char **argv)
{
    return pam_sm_authenticate(pamh, flags, argc, argv);
}

PAM_EXTERN int
pam_sm_close_session (pam_handle_t *pamh, int flags,
		      int argc, const char **argv)
{
    return pam_sm_authenticate(pamh, flags, argc, argv);
}

PAM_EXTERN int
pam_sm_chauthtok (pam_handle_t *pamh, int flags,
		  int argc, const char **argv)
{
    return pam_sm_authenticate(pamh, flags, argc, argv);
}

#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_setitem_modstruct = {
    "pam_setitem",
    pam_sm_authenticate,
    pam_sm_setcred,
    pam_sm_acct_mgmt,
    pam_sm_open_session,
    pam_sm_close_session,
    pam_sm_chauthtok,
};

#endif /* PAM_STATIC */

/* end of module definition */
