/* 
 * Skeleton VFS module.  Implements dummy versions of all VFS
 * functions.
 *
 * Copyright (C) Tim Potter, 1999-2000
 * Copyright (C) Alexander Bokovoy, 2002
 * Copyright (C) Stefan (metze) Metzmacher, 2003
 * Copyright (C) Jeremy Allison 2009
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */


#include "includes.h"
#include "smbd/proto.h"
#include <sys/stat.h>
#include <syslog.h>

/* PLEASE,PLEASE READ THE VFS MODULES CHAPTER OF THE 
   SAMBA DEVELOPERS GUIDE!!!!!!
 */

/* If you take this file as template for your module
 * you must re-implement every function.
 */


static int vfs_nomovetosgid_rename(vfs_handle_struct *handle,
		       const struct smb_filename *smb_fname_src,
		       const struct smb_filename *smb_fname_dst)
{
	struct stat stat_buf;
	char *trgt_file;
	char *parent_dir;
	
	trgt_file = smb_fname_str_dbg(smb_fname_dst);
	parent_dir = dirname(trgt_file);
	if(stat(parent_dir, &stat_buf) == 0 && (stat_buf.st_mode & S_ISGID) && getegid() != stat_buf.st_gid)
	{
		syslog(LOG_DEBUG, "Moving file to SGID directory is disabled: %s -> %s, %d -> %d\n", smb_fname_str_dbg(smb_fname_src), trgt_file, getegid(), stat_buf.st_gid);
		errno = EXDEV; /* upper layer software should fall back to copy&delete mode */
		return -1;
	}
	else
	{
		return SMB_VFS_NEXT_RENAME(handle, smb_fname_src, smb_fname_dst);
	}
}

/* VFS operations structure */

struct vfs_fn_pointers vfs_nomovetosgid_fns = {

	/* File operations */

	.rename = vfs_nomovetosgid_rename,
};

//NTSTATUS vfs_skel_transparent_init(void)
NTSTATUS init_samba_module(void)
{
	return smb_register_vfs(SMB_VFS_INTERFACE_VERSION, "nomovetosgid", &vfs_nomovetosgid_fns);
}
